@@ -1,3 +1,4 @@
+# coding=utf-8
 # Copyright 2015,2016 Nir Cohen
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -191,11 +192,11 @@ class TestOSRelease:
         desired_outcome = {
             'id': 'fedora',
             'name': 'Fedora',
-            'pretty_name': u'Fedora 19 (Schr\u00F6dinger\u2019s Cat)',
+            'pretty_name': 'Fedora 19 (Schrödinger’s Cat)',
             'version': '19',
-            'pretty_version': u'19 (Schr\u00F6dinger\u2019s Cat)',
+            'pretty_version': '19 (Schrödinger’s Cat)',
             'best_version': '19',
-            'codename': u'Schr\u00F6dinger\u2019s Cat'
+            'codename': 'Schrödinger’s Cat'
         }
         self._test_outcome(desired_outcome)
 
@@ -211,6 +212,21 @@ class TestOSRelease:
         }
         self._test_outcome(desired_outcome)
 
+    def test_fedora30_os_release(self):
+        # Fedora 21 and above no longer have code names but the metadata in os-release was only
+        # changed in a detectable way in Fedora 30+.  The piece in parenthesis in the pretty_name
+        # field contains the VARIANT and differs depending on the variant which was installed.
+        desired_outcome = {
+            'id': 'fedora',
+            'name': 'Fedora',
+            'pretty_name': 'Fedora 30 (Thirty)',
+            'version': '30',
+            'pretty_version': '30',
+            'best_version': '30',
+            'codename': ''
+        }
+        self._test_outcome(desired_outcome)
+
     def test_kvmibm1_os_release(self):
         desired_outcome = {
             'id': 'kvmibm',
@@ -344,6 +360,19 @@ class TestOSRelease:
         }
         self._test_outcome(desired_outcome)
 
+    def test_ubuntu16_os_release(self):
+        desired_outcome = {
+            'id': 'ubuntu',
+            'name': 'Ubuntu',
+            'pretty_name': 'Ubuntu 16.04.1 LTS',
+            'version': '16.04',
+            'pretty_version': '16.04 (xenial)',
+            'best_version': '16.04.1',
+            'like': 'debian',
+            'codename': 'xenial'
+        }
+        self._test_outcome(desired_outcome)
+
     def test_amazon2016_os_release(self):
         desired_outcome = {
             'id': 'amzn',
@@ -492,6 +521,16 @@ class TestLSBRelease(DistroTestCase):
             'best_version': '11.1'
         })
 
+    def test_midnightbsd12_uname(self):
+        self._test_outcome({
+            'id': 'midnightbsd',
+            'name': 'MidnightBSD',
+            'version': '1.2',
+            'pretty_name': 'MidnightBSD 1.2',
+            'pretty_version': '1.2',
+            'best_version': '1.2'
+        })
+
     def test_ubuntu14normal_lsb_release(self):
         self._setup_for_distro(os.path.join(TESTDISTROS, 'lsb',
                                             'ubuntu14_normal'))
@@ -675,11 +714,11 @@ class TestDistroRelease:
         desired_outcome = {
             'id': 'fedora',
             'name': 'Fedora',
-            'pretty_name': u'Fedora 19 (Schr\u00F6dinger\u2019s Cat)',
+            'pretty_name': 'Fedora 19 (Schrödinger’s Cat)',
             'version': '19',
-            'pretty_version': u'19 (Schr\u00F6dinger\u2019s Cat)',
+            'pretty_version': '19 (Schrödinger’s Cat)',
             'best_version': '19',
-            'codename': u'Schr\u00F6dinger\u2019s Cat',
+            'codename': 'Schrödinger’s Cat',
             'major_version': '19'
         }
         self._test_outcome(desired_outcome, 'fedora', '19')
@@ -697,6 +736,19 @@ class TestDistroRelease:
         }
         self._test_outcome(desired_outcome, 'fedora', '23')
 
+    def test_fedora30_dist_release(self):
+        desired_outcome = {
+            'id': 'fedora',
+            'name': 'Fedora',
+            'pretty_name': 'Fedora 30 (Thirty)',
+            'version': '30',
+            'pretty_version': '30 (Thirty)',
+            'best_version': '30',
+            'codename': 'Thirty',
+            'major_version': '30'
+        }
+        self._test_outcome(desired_outcome, 'fedora', '30')
+
     def test_gentoo_dist_release(self):
         desired_outcome = {
             'id': 'gentoo',
@@ -765,12 +817,12 @@ class TestDistroRelease:
         desired_outcome = {
             'id': 'oracle',
             'name': 'Oracle Linux Server',
-            'pretty_name': 'Oracle Linux Server 7.1',
-            'version': '7.1',
-            'pretty_version': '7.1',
-            'best_version': '7.1',
+            'pretty_name': 'Oracle Linux Server 7.5',
+            'version': '7.5',
+            'pretty_version': '7.5',
+            'best_version': '7.5',
             'major_version': '7',
-            'minor_version': '1'
+            'minor_version': '5'
         }
         self._test_outcome(desired_outcome, 'oracle', '7')
 
@@ -836,7 +888,7 @@ class TestDistroRelease:
         # Uses redhat-release only to get information.
         # The id of 'rhel' can only be fixed with issue #109.
         desired_outcome = {
-            'id': 'rhel',
+            'id': 'cloudlinux',
             'codename': 'Vladislav Volkov',
             'name': 'CloudLinux Server',
             'pretty_name': 'CloudLinux Server 5.11 (Vladislav Volkov)',
@@ -851,7 +903,7 @@ class TestDistroRelease:
     def test_cloudlinux6_dist_release(self):
         # Same as above, only has redhat-release.
         desired_outcome = {
-            'id': 'rhel',
+            'id': 'cloudlinux',
             'codename': 'Oleg Makarov',
             'name': 'CloudLinux Server',
             'pretty_name': 'CloudLinux Server 6.8 (Oleg Makarov)',
@@ -865,7 +917,7 @@ class TestDistroRelease:
 
     def test_cloudlinux7_dist_release(self):
         desired_outcome = {
-            'id': 'rhel',
+            'id': 'cloudlinux',
             'codename': 'Yury Malyshev',
             'name': 'CloudLinux',
             'pretty_name': 'CloudLinux 7.3 (Yury Malyshev)',
@@ -1037,11 +1089,11 @@ class TestOverall(DistroTestCase):
         desired_outcome = {
             'id': 'fedora',
             'name': 'Fedora',
-            'pretty_name': u'Fedora 19 (Schr\u00F6dinger\u2019s Cat)',
+            'pretty_name': 'Fedora 19 (Schrödinger’s Cat)',
             'version': '19',
-            'pretty_version': u'19 (Schr\u00F6dinger\u2019s Cat)',
+            'pretty_version': '19 (Schrödinger’s Cat)',
             'best_version': '19',
-            'codename': u'Schr\u00F6dinger\u2019s Cat',
+            'codename': 'Schrödinger’s Cat',
             'major_version': '19'
         }
         self._test_outcome(desired_outcome)
@@ -1050,7 +1102,7 @@ class TestOverall(DistroTestCase):
             'id': 'fedora',
             'name': 'Fedora',
             'version_id': '19',
-            'codename': u'Schr\u00F6dinger\u2019s Cat'
+            'codename': 'Schrödinger’s Cat'
         }
         self._test_release_file_info('fedora-release', desired_info)
 
@@ -1075,6 +1127,27 @@ class TestOverall(DistroTestCase):
         }
         self._test_release_file_info('fedora-release', desired_info)
 
+    def test_fedora30_release(self):
+        desired_outcome = {
+            'id': 'fedora',
+            'name': 'Fedora',
+            'pretty_name': 'Fedora 30 (Thirty)',
+            'version': '30',
+            'pretty_version': '30',
+            'best_version': '30',
+            'codename': '',
+            'major_version': '30'
+        }
+        self._test_outcome(desired_outcome)
+
+        desired_info = {
+            'id': 'fedora',
+            'name': 'Fedora',
+            'version_id': '30',
+            'codename': 'Thirty'
+        }
+        self._test_release_file_info('fedora-release', desired_info)
+
     def test_kvmibm1_release(self):
         desired_outcome = {
             'id': 'kvmibm',
@@ -1183,19 +1256,19 @@ class TestOverall(DistroTestCase):
         desired_outcome = {
             'id': 'oracle',
             'name': 'Oracle Linux Server',
-            'pretty_name': 'Oracle Linux Server 7.1',
-            'version': '7.1',
-            'pretty_version': '7.1',
-            'best_version': '7.1',
+            'pretty_name': 'Oracle Linux Server 7.5',
+            'version': '7.5',
+            'pretty_version': '7.5',
+            'best_version': '7.5',
             'major_version': '7',
-            'minor_version': '1'
+            'minor_version': '5'
         }
         self._test_outcome(desired_outcome)
 
         desired_info = {
             'id': 'oracle',
             'name': 'Oracle Linux Server',
-            'version_id': '7.1',
+            'version_id': '7.5',
         }
         distro_info = self._test_release_file_info(
             'oracle-release', desired_info)
@@ -1362,6 +1435,26 @@ class TestOverall(DistroTestCase):
         # release file:
         self._test_non_existing_release_file()
 
+    def test_ubuntu16_release(self):
+        desired_outcome = {
+            'id': 'ubuntu',
+            'name': 'Ubuntu',
+            'pretty_name': 'Ubuntu 16.04.1 LTS',
+            'version': '16.04',
+            'pretty_version': '16.04 (xenial)',
+            'best_version': '16.04.1',
+            'like': 'debian',
+            'codename': 'xenial',
+            'major_version': '16',
+            'minor_version': '04'
+        }
+        self._test_outcome(desired_outcome)
+
+        # Test the info from the searched distro release file
+        # Does not have one; /etc/debian_version is not considered a distro
+        # release file:
+        self._test_non_existing_release_file()
+
     def test_amazon2016_release(self):
         desired_outcome = {
             'id': 'amzn',
@@ -1486,7 +1579,7 @@ class TestOverall(DistroTestCase):
         # Uses redhat-release only to get information.
         # The id of 'rhel' can only be fixed with issue #109.
         desired_outcome = {
-            'id': 'rhel',
+            'id': 'cloudlinux',
             'codename': 'Vladislav Volkov',
             'name': 'CloudLinux Server',
             'pretty_name': 'CloudLinux Server 5.11 (Vladislav Volkov)',
@@ -1501,7 +1594,7 @@ class TestOverall(DistroTestCase):
     def test_cloudlinux6_release(self):
         # Same as above, only has redhat-release.
         desired_outcome = {
-            'id': 'rhel',
+            'id': 'cloudlinux',
             'codename': 'Oleg Makarov',
             'name': 'CloudLinux Server',
             'pretty_name': 'CloudLinux Server 6.8 (Oleg Makarov)',
@@ -1536,7 +1629,7 @@ def _bad_os_listdir(path='.'):
     raise OSError()
 
 
-@pytest.mark.skipIf(not IS_LINUX, reason='Irrelevant on non-linx')
+@pytest.mark.skipif(not IS_LINUX, reason='Irrelevant on non-linux')
 class TestOverallWithEtcNotReadable(TestOverall):
     def setup_method(self, test_method):
         self._old_listdir = os.listdir
@@ -1655,12 +1748,12 @@ class TestInfo(DistroTestCase):
         info = _distro.info(pretty=True, best=True)
         _test_none(info)
 
-    def test_linux_disribution(self):
+    def test_linux_distribution(self):
         _distro = distro.LinuxDistribution(False, self.ubuntu14_os_release)
         i = _distro.linux_distribution()
         assert i == ('Ubuntu', '14.04', 'Trusty Tahr')
 
-    def test_linux_disribution_full_false(self):
+    def test_linux_distribution_full_false(self):
         _distro = distro.LinuxDistribution(False, self.ubuntu14_os_release)
         i = _distro.linux_distribution(full_distribution_name=False)
         assert i == ('ubuntu', '14.04', 'Trusty Tahr')
@@ -1978,3 +2071,20 @@ class TestRepr:
         assert "LinuxDistribution" in repr_str
         for attr in MODULE_DISTRO.__dict__.keys():
             assert attr + '=' in repr_str
+
+
+@pytest.mark.skipif(not IS_LINUX, reason='Irrelevant on non-linux')
+class TestToStr:
+    """Test the _to_str() method.
+    """
+
+    def test_to_str(self):
+        ret = distro.LinuxDistribution._to_str(b'bytes')
+        assert isinstance(ret, str)
+        assert ret == 'bytes'
+
+        ret = distro.LinuxDistribution._to_str(u'bytes')
+        assert isinstance(ret, str)
+
+        ret = distro.LinuxDistribution._to_str('bytes')
+        assert isinstance(ret, str)
